
from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    Cloud,
    CloudBulkRequest,
    CloudBulkResponse,
    CloudQueryConfig,
    CloudQueryResponse,
)


class CloudService(BaseService):

    @cast_models
    def get_cloud(self, id_: str) -> Union[Cloud, str]:
        """Retrieves the properties of the Runtime cloud having the specified ID.

        :param id_: A unique ID assigned by the system to the Runtime cloud.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Cloud, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Cloud/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Cloud._unmap(response)
        if content == "application/xml":
            return Cloud._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_cloud(
        self, request_body: CloudBulkRequest = None
    ) -> Union[CloudBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: CloudBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[CloudBulkResponse, str]
        """

        Validator(CloudBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Cloud/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return CloudBulkResponse._unmap(response)
        if content == "application/xml":
            return CloudBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_cloud(
        self, request_body: CloudQueryConfig = None
    ) -> Union[CloudQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: CloudQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[CloudQueryResponse, str]
        """

        Validator(CloudQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Cloud/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return CloudQueryResponse._unmap(response)
        if content == "application/xml":
            return CloudQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_cloud(self, request_body: str) -> Union[CloudQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[CloudQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Cloud/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return CloudQueryResponse._unmap(response)
        if content == "application/xml":
            return CloudQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ExecutionRecordSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    :cvar LIKE: "LIKE"
    :vartype LIKE: str
    :cvar NOTEQUALS: "NOT_EQUALS"
    :vartype NOTEQUALS: str
    :cvar ISNULL: "IS_NULL"
    :vartype ISNULL: str
    :cvar ISNOTNULL: "IS_NOT_NULL"
    :vartype ISNOTNULL: str
    :cvar BETWEEN: "BETWEEN"
    :vartype BETWEEN: str
    :cvar GREATERTHAN: "GREATER_THAN"
    :vartype GREATERTHAN: str
    :cvar GREATERTHANOREQUAL: "GREATER_THAN_OR_EQUAL"
    :vartype GREATERTHANOREQUAL: str
    :cvar LESSTHAN: "LESS_THAN"
    :vartype LESSTHAN: str
    :cvar LESSTHANOREQUAL: "LESS_THAN_OR_EQUAL"
    :vartype LESSTHANOREQUAL: str
    :cvar CONTAINS: "CONTAINS"
    :vartype CONTAINS: str
    :cvar NOTCONTAINS: "NOT_CONTAINS"
    :vartype NOTCONTAINS: str
    """

    EQUALS = "EQUALS"
    LIKE = "LIKE"
    NOTEQUALS = "NOT_EQUALS"
    ISNULL = "IS_NULL"
    ISNOTNULL = "IS_NOT_NULL"
    BETWEEN = "BETWEEN"
    GREATERTHAN = "GREATER_THAN"
    GREATERTHANOREQUAL = "GREATER_THAN_OR_EQUAL"
    LESSTHAN = "LESS_THAN"
    LESSTHANOREQUAL = "LESS_THAN_OR_EQUAL"
    CONTAINS = "CONTAINS"
    NOTCONTAINS = "NOT_CONTAINS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                ExecutionRecordSimpleExpressionOperator._member_map_.values(),
            )
        )


class ExecutionRecordSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar EXECUTIONID: "executionId"
    :vartype EXECUTIONID: str
    :cvar ORIGINALEXECUTIONID: "originalExecutionId"
    :vartype ORIGINALEXECUTIONID: str
    :cvar ACCOUNT: "account"
    :vartype ACCOUNT: str
    :cvar EXECUTIONTIME: "executionTime"
    :vartype EXECUTIONTIME: str
    :cvar STATUS: "status"
    :vartype STATUS: str
    :cvar EXECUTIONTYPE: "executionType"
    :vartype EXECUTIONTYPE: str
    :cvar PROCESSNAME: "processName"
    :vartype PROCESSNAME: str
    :cvar PROCESSID: "processId"
    :vartype PROCESSID: str
    :cvar ATOMNAME: "atomName"
    :vartype ATOMNAME: str
    :cvar ATOMID: "atomId"
    :vartype ATOMID: str
    :cvar INBOUNDDOCUMENTCOUNT: "inboundDocumentCount"
    :vartype INBOUNDDOCUMENTCOUNT: str
    :cvar OUTBOUNDDOCUMENTCOUNT: "outboundDocumentCount"
    :vartype OUTBOUNDDOCUMENTCOUNT: str
    :cvar EXECUTIONDURATION: "executionDuration"
    :vartype EXECUTIONDURATION: str
    :cvar MESSAGE: "message"
    :vartype MESSAGE: str
    :cvar REPORTKEY: "reportKey"
    :vartype REPORTKEY: str
    :cvar LAUNCHERID: "launcherId"
    :vartype LAUNCHERID: str
    :cvar NODEID: "nodeId"
    :vartype NODEID: str
    :cvar RECORDEDDATE: "recordedDate"
    :vartype RECORDEDDATE: str
    """

    EXECUTIONID = "executionId"
    ORIGINALEXECUTIONID = "originalExecutionId"
    ACCOUNT = "account"
    EXECUTIONTIME = "executionTime"
    STATUS = "status"
    EXECUTIONTYPE = "executionType"
    PROCESSNAME = "processName"
    PROCESSID = "processId"
    ATOMNAME = "atomName"
    ATOMID = "atomId"
    INBOUNDDOCUMENTCOUNT = "inboundDocumentCount"
    OUTBOUNDDOCUMENTCOUNT = "outboundDocumentCount"
    EXECUTIONDURATION = "executionDuration"
    MESSAGE = "message"
    REPORTKEY = "reportKey"
    LAUNCHERID = "launcherId"
    NODEID = "nodeId"
    RECORDEDDATE = "recordedDate"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                ExecutionRecordSimpleExpressionProperty._member_map_.values(),
            )
        )


@JsonMap({})
class ExecutionRecordSimpleExpression(BaseModel):
    """ExecutionRecordSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: ExecutionRecordSimpleExpressionOperator
    :param property: property
    :type property: ExecutionRecordSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: ExecutionRecordSimpleExpressionOperator,
        property: ExecutionRecordSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """ExecutionRecordSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: ExecutionRecordSimpleExpressionOperator
        :param property: property
        :type property: ExecutionRecordSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator, ExecutionRecordSimpleExpressionOperator.list(), "operator"
        )
        self.property = self._enum_matching(
            property, ExecutionRecordSimpleExpressionProperty.list(), "property"
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"status_code": "statusCode"})
class LogDownload(BaseModel):
    """LogDownload

    :param status_code: The status code as one of the following:   - 202 — status message: Beginning download.  - 504 — status message: Runtime is unavailable., defaults to None
    :type status_code: str, optional
    :param message: The status message., defaults to None
    :type message: str, optional
    :param url: (statusCode 202 only) The URL for the download., defaults to None
    :type url: str, optional
    """

    def __init__(
        self,
        status_code: str = SENTINEL,
        message: str = SENTINEL,
        url: str = SENTINEL,
        **kwargs
    ):
        """LogDownload

        :param status_code: The status code as one of the following:   - 202 — status message: Beginning download.  - 504 — status message: Runtime is unavailable., defaults to None
        :type status_code: str, optional
        :param message: The status message., defaults to None
        :type message: str, optional
        :param url: (statusCode 202 only) The URL for the download., defaults to None
        :type url: str, optional
        """
        if status_code is not SENTINEL:
            self.status_code = status_code
        if message is not SENTINEL:
            self.message = message
        if url is not SENTINEL:
            self.url = url
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .listener_status import ListenerStatus


class ResponseStatusCode(Enum):
    """An enumeration representing different categories.

    :cvar _200: 200
    :vartype _200: int
    :cvar _202: 202
    :vartype _202: int
    """

    _200 = 200
    _202 = 202

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ResponseStatusCode._member_map_.values()))


@JsonMap(
    {
        "number_of_results": "numberOfResults",
        "response_status_code": "responseStatusCode",
    }
)
class ListenerStatusAsyncResponse(BaseModel):
    """ListenerStatusAsyncResponse

    :param number_of_results: number_of_results, defaults to None
    :type number_of_results: int, optional
    :param response_status_code: The status code returned from a request, as follows:   - 202 — Initialized the Listener status request and is in progress (QUERY response).  - 200 — Listener status request is complete (GET response).
    :type response_status_code: ResponseStatusCode
    :param result: result, defaults to None
    :type result: List[ListenerStatus], optional
    """

    def __init__(
        self,
        response_status_code: ResponseStatusCode,
        number_of_results: int = SENTINEL,
        result: List[ListenerStatus] = SENTINEL,
        **kwargs,
    ):
        """ListenerStatusAsyncResponse

        :param number_of_results: number_of_results, defaults to None
        :type number_of_results: int, optional
        :param response_status_code: The status code returned from a request, as follows:   - 202 — Initialized the Listener status request and is in progress (QUERY response).  - 200 — Listener status request is complete (GET response).
        :type response_status_code: ResponseStatusCode
        :param result: result, defaults to None
        :type result: List[ListenerStatus], optional
        """
        if number_of_results is not SENTINEL:
            self.number_of_results = number_of_results
        self.response_status_code = self._enum_matching(
            response_status_code, ResponseStatusCode.list(), "response_status_code"
        )
        if result is not SENTINEL:
            self.result = self._define_list(result, ListenerStatus)
        self._kwargs = kwargs
